#!/usr/bin/perl -w 

package esmith::FormMagick::Panel::ejabberd;

use strict;

use esmith::FormMagick;
use esmith::ConfigDB;
use esmith::DomainsDB;
use File::Basename;
use Exporter;
use Carp;

our @ISA = qw(esmith::FormMagick Exporter);

our @EXPORT = qw(

  show_initial
  get_ejabberd_access
  get_ejabberd_domain
  get_ejabberd_register
  get_ejabberd_xmppaccess
  get_ejabberd_webadmin
  get_ejabberd_welcome_subject
  get_ejabberd_welcome_text
  get_cgi_param
  find_domains
);

our $db = esmith::ConfigDB->open() || die "Couldn't open config db";
our $domains = esmith::DomainsDB->open_ro() || die "Couldn't open domains";

our $VERSION = sprintf '%d.%03d', q$Revision: 2.00 $ =~ /: (\d+).(\d+)/;


=pod 

=head1 NAME

esmith::FormMagick::Panels::ejabberd - useful panel functions

=head1 SYNOPSIS

    use esmith::FormMagick::Panels::ejabberd;

    my $panel = esmith::FormMagick::Panel::ejabberd->new();
    $panel->display();

=head1 DESCRIPTION

=cut


=head2 new();

Exactly as for esmith::FormMagick

=begin testing

$ENV{ESMITH_DOMAINS_DB} = "10e-smith-base/domains.conf";
$ENV{ESMITH_CONFIG_DB} = "10e-smith-base/configuration.conf";

use_ok('esmith::FormMagick::Panel::ejabberd');
use vars qw($panel);
ok($panel = esmith::FormMagick::Panel::ejabberd->new(), "Create panel object");
isa_ok($panel, 'esmith::FormMagick::Panel::ejabberd');

=end testing

=cut

sub new {
    shift;
    my $self = esmith::FormMagick->new();
    $self->{calling_package} = (caller)[0];
    bless $self;
    return $self;
}


=head1 ACCESSORS

=head2 get_cgi_param FM FIELD

Returns the named CGI parameter as a string

=cut

sub get_cgi_param {
    my $fm    = shift;
    my $param = shift;

    return ( $fm->{'cgi'}->param($param) );
}


=head2 get_prop ITEM PROP

A simple accessor for esmith::ConfigDB::Record::prop

=cut

sub get_prop {
  my ($self, $item, $prop) = @_;
  warn "You must specify a record key"    unless $item;
  warn "You must specify a property name" unless $prop;
  my $record = $db->get($item) or warn "Couldn't get record for $item";
  return $record ? $record->prop($prop) : undef;
}

=head2 get_value ITEM

A simple accessor for esmith::ConfigDB::Record::value

=cut

sub get_value {
  my $self = shift;
  my $item = shift;
  return ($db->get($item)->value());
}

=head2 get_ejabberd_access

Returns "public", "private" or "off" depending on the 'access' and 'status' properties
of the "ejabberd" config file variable

=cut

sub get_ejabberd_access {

  my $status = get_prop('','ejabberd','status');
  if ( (defined $status) && ($status eq 'enabled')) {
    my $access = get_prop('','ejabberd','access');
    return $access ? $access : 'off';
  }
  else {
    return('off');
  }
}

=head2 get_ejabberd_domain

Returns value of 'domain' property of the "ejabberd" config file variable

=cut

sub get_ejabberd_domain
{
    return(get_prop('$self','ejabberd','Domain') || $db->get('DomainName')->value);
}

=head2 get_ejabberd_register

Returns value of 'Register' property of the "ejabberd" config file variable

=cut

sub get_ejabberd_register
{
    return(get_prop('$self','ejabberd','Register') || 'smeusers');
}

=head2 get_ejabberd_xmppaccess

Returns value of 'XMPPAccess' property of the "ejabberd" config file variable

=cut

sub get_ejabberd_xmppaccess
{
    return(get_prop('$self','ejabberd','XMPPAccess') || 'notls');
}

=head2 get_ejabberd_webadmin

Returns value of 'WebAdmin' property of the "ejabberd" config file variable

=cut

sub get_ejabberd_webadmin
{
    return(get_prop('$self','ejabberd','WebAdmin') || 'disabled');
}

=head2 get_ejabberd_welcome_subject

Returns value of 'WelcomeSubject' property of the "ejabberd" config file variable

=cut

sub get_ejabberd_welcome_subject
{
    return(get_prop('$self','ejabberd','WelcomeSubject') || '');
}

=head2 get_ejabberd_welcome_text

Returns value of 'WelcomeText' property of the "ejabberd" config file variable

=cut

sub get_ejabberd_welcome_text
{
    return(get_prop('$self','ejabberd','WelcomeText') || '');
}

=head2 find_domains

Returns list of all declared domains names from "domains" config file

=cut

sub find_domains
{
    my %domainslist = ();
    my $d;
    foreach $d ($domains->domains()) {
	$domainslist{$d->key()} = $d->key();
	}
    return \%domainslist;
}


=head1 ACTION


=head2 build_cgi_params()

Builds a CGI query string, using various sensible
defaults and esmith::FormMagick's props_to_query_string() method.

=cut

sub build_cgi_params {
    my ( $fm, $group ) = @_;

    my %props = (
        page       => 0,
        page_stack => "",
        ".id"      => $fm->{cgi}->param('.id') || "",
        groupName  => $group,
    );

    return $fm->props_to_query_string( \%props );
}

=head2 change_settings

If everything has been validated, properly, go ahead and set the new settings

=cut


sub change_settings {
    my ($self) = @_;

    my %conf;

    my $q = $self->{'cgi'};

	# Don't process the form unless we clicked the Save button. The event is
	# called even if we chose the Remove link or the Add link.
	return unless($q->param('Next') eq $self->localise('SAVE'));

    my $access = ($q->param ('access') || 'off');
    my $webadmin = ($q->param ('webadmin') || 'disabled');
    my $register = ($q->param ('register') || 'smeusers');
    my $xmppaccess = ($q->param ('xmppaccess') || 'notls');
    my $domain = ($q->param ('domain') || 'localhost');
    my $welcome_subject = ($q->param ('welcome_subject') || '');
    my $welcome_text = $welcome_subject ? $q->param ('welcome_text') : '';

    #------------------------------------------------------------
    # Looks good; go ahead and change the access.
    #------------------------------------------------------------

    my $rec = $db->get('ejabberd');
    if($rec)
    {
    	if ($access eq "off")
    	{
	    $rec->set_prop('status','disabled');
    	}
    	else
      	{
	    $rec->set_prop('status','enabled');
	    $rec->set_prop('access', $access);
      	}
    }
    $rec->set_prop('WebAdmin', $webadmin);
    $rec->set_prop('Register', $register);
    $rec->set_prop('XMPPAccess', $xmppaccess);
    $rec->set_prop('Domain', $domain);
    $rec->set_prop('WelcomeSubject', $welcome_subject);
    $rec->set_prop('WelcomeText', $welcome_text);

    $self->cgi->param(-name=>'wherenext', -value=>'First');

    unless ( system( "/sbin/e-smith/signal-event", "ejabberd-update" ) == 0 )
    {
        $self->error('ERROR_UPDATING');
        return undef;
    }

    $self->success('SUCCESS');
}

=head2 clear_params

This method clears-out the parameters used in form submission so that they are
not inadvertenly picked-up where they should not be.

=cut

sub clear_params
{
    my $self = shift;
    my $q = $self->{cgi};

    $q->delete('groupMembers');
    $q->delete('groupDesc');
    $q->delete('groupName');
}
